@@ -38,6 +38,7 @@ from .path import CompoundJSONPath
 from .path import JSONPath
 from .stream import TokenStream
 from .token import TOKEN_EOF
+from .token import TOKEN_FAKE_ROOT
 from .token import TOKEN_INTERSECTION
 from .token import TOKEN_UNION
 from .token import Token
@@ -89,6 +90,8 @@ class JSONPathEnvironment:
             **New in version 0.10.0**
 
     Attributes:
+        fake_root_token (str): The pattern used to select a "fake" root node, one level
+            above the real root node.
         filter_context_token (str): The pattern used to select extra filter context
             data. Defaults to `"_"`.
         intersection_token (str): The pattern used as the intersection operator.
@@ -112,6 +115,7 @@ class JSONPathEnvironment:
 
     # These should be unescaped strings. `re.escape` will be called
     # on them automatically when compiling lexer rules.
+    fake_root_token = "^"
     filter_context_token = "_"
     intersection_token = "&"
     key_token = "#"
@@ -174,8 +178,9 @@ class JSONPathEnvironment:
         """
         tokens = self.lexer.tokenize(path)
         stream = TokenStream(tokens)
+        fake_root = stream.current.kind == TOKEN_FAKE_ROOT
         _path: Union[JSONPath, CompoundJSONPath] = JSONPath(
-            env=self, selectors=self.parser.parse(stream)
+            env=self, selectors=self.parser.parse(stream), fake_root=fake_root
         )
 
         if stream.current.kind != TOKEN_EOF:
@@ -190,18 +195,22 @@ class JSONPathEnvironment:
 
                 if stream.current.kind == TOKEN_UNION:
                     stream.next_token()
+                    fake_root = stream.current.kind == TOKEN_FAKE_ROOT
                     _path = _path.union(
                         JSONPath(
                             env=self,
                             selectors=self.parser.parse(stream),
+                            fake_root=fake_root,
                         )
                     )
                 elif stream.current.kind == TOKEN_INTERSECTION:
                     stream.next_token()
+                    fake_root = stream.current.kind == TOKEN_FAKE_ROOT
                     _path = _path.intersection(
                         JSONPath(
                             env=self,
                             selectors=self.parser.parse(stream),
+                            fake_root=fake_root,
                         )
                     )
                 else:  # pragma: no cover
@@ -327,7 +336,6 @@ class JSONPathEnvironment:
 
     def setup_function_extensions(self) -> None:
         """Initialize function extensions."""
-        self.function_extensions["keys"] = function_extensions.keys
         self.function_extensions["length"] = function_extensions.Length()
         self.function_extensions["count"] = function_extensions.Count()
         self.function_extensions["match"] = function_extensions.Match()
@@ -343,8 +351,8 @@ class JSONPathEnvironment:
     ) -> List[Any]:
         """Compile-time validation of function extension arguments.
 
-        The IETF JSONPath draft requires us to reject paths that use filter
-        functions with too many or too few arguments.
+        RFC 9535 requires us to reject paths that use filter functions with
+        too many or too few arguments.
         """
         try:
             func = self.function_extensions[token.value]
@@ -443,10 +451,10 @@ class JSONPathEnvironment:
     def is_truthy(self, obj: object) -> bool:
         """Test for truthiness when evaluating JSONPath filter expressions.
 
-        In some cases, the IETF JSONPath draft requires us to test for
-        existence rather than truthiness. So the default implementation returns
-        `True` for empty collections and `None`. The special `UNDEFINED` object
-        means that _obj_ was missing, as opposed to an explicit `None`.
+        In some cases, RFC 9535 requires us to test for existence rather than
+        truthiness. So the default implementation returns `True` for empty
+        collections and `None`. The special `UNDEFINED` object means that
+        _obj_ was missing, as opposed to an explicit `None`.
 
         Arguments:
             obj: Any object.
