@@ -50,7 +50,7 @@ class JSONPathSelector(ABC):
 class PropertySelector(JSONPathSelector):
     """A JSONPath property."""
 
-    __slots__ = ("name",)
+    __slots__ = ("name", "shorthand")
 
     def __init__(
         self,
@@ -58,12 +58,24 @@ class PropertySelector(JSONPathSelector):
         env: JSONPathEnvironment,
         token: Token,
         name: str,
+        shorthand: bool,
     ) -> None:
         super().__init__(env=env, token=token)
         self.name = name
+        self.shorthand = shorthand
 
     def __str__(self) -> str:
-        return f"['{self.name}']"
+        return f"['{self.name}']" if self.shorthand else f"'{self.name}'"
+
+    def __eq__(self, __value: object) -> bool:
+        return (
+            isinstance(__value, PropertySelector)
+            and self.name == __value.name
+            and self.token == __value.token
+        )
+
+    def __hash__(self) -> int:
+        return hash((self.name, self.token))
 
     def resolve(self, matches: Iterable[JSONPathMatch]) -> Iterable[JSONPathMatch]:
         for match in matches:
@@ -122,7 +134,17 @@ class IndexSelector(JSONPathSelector):
         self._as_key = str(self.index)
 
     def __str__(self) -> str:
-        return f"[{self.index}]"
+        return str(self.index)
+
+    def __eq__(self, __value: object) -> bool:
+        return (
+            isinstance(__value, IndexSelector)
+            and self.index == __value.index
+            and self.token == __value.token
+        )
+
+    def __hash__(self) -> int:
+        return hash((self.index, self.token))
 
     def _normalized_index(self, obj: Sequence[object]) -> int:
         if self.index < 0 and len(obj) >= abs(self.index):
@@ -144,7 +166,7 @@ class IndexSelector(JSONPathSelector):
                     )
                     match.add_child(_match)
                     yield _match
-            elif isinstance(match.obj, Sequence):
+            elif isinstance(match.obj, Sequence) and not isinstance(match.obj, str):
                 norm_index = self._normalized_index(match.obj)
                 with suppress(IndexError):
                     _match = self.env.match_class(
@@ -175,7 +197,7 @@ class IndexSelector(JSONPathSelector):
                     )
                     match.add_child(_match)
                     yield _match
-            elif isinstance(match.obj, Sequence):
+            elif isinstance(match.obj, Sequence) and not isinstance(match.obj, str):
                 norm_index = self._normalized_index(match.obj)
                 with suppress(IndexError):
                     _match = self.env.match_class(
@@ -193,10 +215,26 @@ class IndexSelector(JSONPathSelector):
 class KeysSelector(JSONPathSelector):
     """Select an mapping's keys/properties."""
 
-    __slots__ = ()
+    __slots__ = ("shorthand",)
+
+    def __init__(
+        self, *, env: JSONPathEnvironment, token: Token, shorthand: bool
+    ) -> None:
+        super().__init__(env=env, token=token)
+        self.shorthand = shorthand
 
     def __str__(self) -> str:
-        return f"[{self.env.keys_selector_token}]"
+        return (
+            f"[{self.env.keys_selector_token}]"
+            if self.shorthand
+            else self.env.keys_selector_token
+        )
+
+    def __eq__(self, __value: object) -> bool:
+        return isinstance(__value, KeysSelector) and self.token == __value.token
+
+    def __hash__(self) -> int:
+        return hash(self.token)
 
     def _keys(self, match: JSONPathMatch) -> Iterable[JSONPathMatch]:
         if isinstance(match.obj, Mapping):
@@ -246,7 +284,17 @@ class SliceSelector(JSONPathSelector):
         stop = self.slice.stop if self.slice.stop is not None else ""
         start = self.slice.start if self.slice.start is not None else ""
         step = self.slice.step if self.slice.step is not None else "1"
-        return f"[{start}:{stop}:{step}]"
+        return f"{start}:{stop}:{step}"
+
+    def __eq__(self, __value: object) -> bool:
+        return (
+            isinstance(__value, SliceSelector)
+            and self.slice == __value.slice
+            and self.token == __value.token
+        )
+
+    def __hash__(self) -> int:
+        return hash((str(self), self.token))
 
     def _check_range(self, *indices: Optional[int]) -> None:
         for i in indices:
@@ -308,8 +356,22 @@ class SliceSelector(JSONPathSelector):
 class WildSelector(JSONPathSelector):
     """Wildcard expansion selector."""
 
+    __slots__ = ("shorthand",)
+
+    def __init__(
+        self, *, env: JSONPathEnvironment, token: Token, shorthand: bool
+    ) -> None:
+        super().__init__(env=env, token=token)
+        self.shorthand = shorthand
+
     def __str__(self) -> str:
-        return "[*]"
+        return "[*]" if self.shorthand else "*"
+
+    def __eq__(self, __value: object) -> bool:
+        return isinstance(__value, WildSelector) and self.token == __value.token
+
+    def __hash__(self) -> int:
+        return hash(self.token)
 
     def resolve(self, matches: Iterable[JSONPathMatch]) -> Iterable[JSONPathMatch]:
         for match in matches:
@@ -376,6 +438,15 @@ class RecursiveDescentSelector(JSONPathSelector):
     def __str__(self) -> str:
         return ".."
 
+    def __eq__(self, __value: object) -> bool:
+        return (
+            isinstance(__value, RecursiveDescentSelector)
+            and self.token == __value.token
+        )
+
+    def __hash__(self) -> int:
+        return hash(self.token)
+
     def _expand(self, match: JSONPathMatch) -> Iterable[JSONPathMatch]:
         if isinstance(match.obj, Mapping):
             for key, val in match.obj.items():
@@ -449,29 +520,25 @@ class ListSelector(JSONPathSelector):
                 IndexSelector,
                 PropertySelector,
                 WildSelector,
+                Filter,
             ]
         ],
     ) -> None:
         super().__init__(env=env, token=token)
-        self.items = items
+        self.items = tuple(items)
 
     def __str__(self) -> str:
-        buf: List[str] = []
-        for item in self.items:
-            if isinstance(item, SliceSelector):
-                stop = item.slice.stop if item.slice.stop is not None else ""
-                start = item.slice.start if item.slice.start is not None else ""
-                step = item.slice.step if item.slice.step is not None else "1"
-                buf.append(f"{start}:{stop}:{step}")
-            elif isinstance(item, PropertySelector):
-                buf.append(f"'{item.name}'")
-            elif isinstance(item, WildSelector):
-                buf.append("*")
-            elif isinstance(item, KeysSelector):
-                buf.append(self.env.keys_selector_token)
-            else:
-                buf.append(str(item.index))
-        return f"[{', '.join(buf)}]"
+        return f"[{', '.join(str(itm) for itm in self.items)}]"
+
+    def __eq__(self, __value: object) -> bool:
+        return (
+            isinstance(__value, ListSelector)
+            and self.items == __value.items
+            and self.token == __value.token
+        )
+
+    def __hash__(self) -> int:
+        return hash((self.items, self.token))
 
     def resolve(self, matches: Iterable[JSONPathMatch]) -> Iterable[JSONPathMatch]:
         _matches = list(matches)
@@ -505,7 +572,17 @@ class Filter(JSONPathSelector):
         self.cacheable_nodes = self.expression.cacheable_nodes()
 
     def __str__(self) -> str:
-        return f"[?({self.expression})]"
+        return f"?{self.expression}"
+
+    def __eq__(self, __value: object) -> bool:
+        return (
+            isinstance(__value, Filter)
+            and self.expression == __value.expression
+            and self.token == __value.token
+        )
+
+    def __hash__(self) -> int:
+        return hash((str(self.expression), self.token))
 
     def resolve(  # noqa: PLR0912
         self, matches: Iterable[JSONPathMatch]
